/*
  Move home the scan slider of GT6801 and GT6816 scanners.
  
  (C) 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* 2004-11-09 */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <errno.h>
#include "usb.h"

#define TIMEOUT 3000

struct id_struct
{
  int vendor;
  int product;
};

static struct id_struct ids[] = {
  {0x055f, 0x0218},
  {0x055f, 0x0219},
  {0x055f, 0x021c},
  {0x055f, 0x021b}, /* Different id? New */
  {0x055f, 0x021d},
  {0x055f, 0x021e},
  {0x055f, 0x021f},
  {0x05d8, 0x4002},
  {0x055f, 0x0210},
  {0x055f, 0x021a},
  {0x043d, 0x002d},
  {0x07b3, 0x0400},
  {0x07b3, 0x0401}, /* Same scanner, different id? */
  {0x07b3, 0x0402},
  {0x07b3, 0x0403}, /* two ids? 403 seems to be more common */
  {0x07b3, 0x0413},
  {0x07b3, 0x0422},
  {0x0458, 0x2011},
  {0x0458, 0x2014}, 
  {0x0458, 0x2017},
  {0x0458, 0x201a},
  {0x0458, 0x201b},
  {0x0458, 0x201f},
  {0x04a7, 0x0444},
  {0, 0}
};

static void
send_return (struct usb_device * dev)
{
  usb_dev_handle * handle;
  char req[64], small_req[8];
  int i;

  printf ("Trying device 0x%04x:0x%04x: ",
	  dev->descriptor.idVendor, dev->descriptor.idProduct);

  if ((handle = usb_open (dev)) == 0)
    {
      printf ("Couldn't open device: %s\n", usb_strerror ());
      return;
    }

  if ((usb_set_configuration (handle, dev->config[0].bConfigurationValue)) < 0)
    {
      printf ("Couldn't set configuration: %s\n", usb_strerror ());
      usb_close (handle);
      return;
    }

  if ((usb_claim_interface (handle, 0)) < 0)
    {
      printf ("Couldn't claim interface: %s\n", usb_strerror ());
      usb_close (handle);
      return;
    }

  memset (small_req, 0, 8);
  small_req[0] = 0x41;
  small_req[1] = 0x01;

  for (i = 0; i < 8; ++i)
    memcpy (req + i * 8, small_req, 8);

  if ((usb_control_msg (handle, 0x40, 0x01, 0x2012, 0x3f40, req, 64, TIMEOUT)) <= 0)
    {
      printf ("Couldn't send write control message (%s)\n",
		strerror (errno));
      usb_release_interface (handle, 0);
      usb_close (handle);
      return;
    }
  if ((usb_control_msg (handle, 0xc0, 0x01, 0x2013, 0x3f00, req, 64, TIMEOUT)) <= 0)
    {
      printf ("Couldn't send read control message (%s)\n",
	      strerror (errno));
      usb_release_interface (handle, 0);
      usb_close (handle);
      return;
    }
  if (req[0] != 0 || (req[1] != 0x41))
    {
      printf ("Unexpected result from control message (%0x/%0x)\n",
		req[0], req[1]);
      usb_release_interface (handle, 0);
      usb_close (handle);
      return;
    }

  memset (req, 0, 64);
  req[0] = 0x24;
  req[1] = 0x01;

  if ((usb_control_msg (handle, 0x40, 0x01, 0x2010, 0x3f40, req, 64, TIMEOUT)) <= 0)
    {
      printf ("Couldn't send write control message (%s)\n",
		strerror (errno));
      usb_release_interface (handle, 0);
      usb_close (handle);
      return;
    }
  if ((usb_control_msg (handle, 0xc0, 0x01, 0x2011, 0x3f00, req, 64, TIMEOUT)) <= 0)
    {
      printf ("Couldn't send read control message (%s)\n",
	      strerror (errno));
      usb_release_interface (handle, 0);
      usb_close (handle);
      return;
    }
  if (req[0] != 0 || (req[1] != 0x24))
    {
      printf ("Unexpected result from control message (%0x/%0x)\n",
		req[0], req[1]);
      usb_release_interface (handle, 0);
      usb_close (handle);
      return;
    }


  usb_release_interface (handle, 0);
  usb_close (handle);

  printf ("\"carriage home\" command succesfully sent.\n");
}

int
main ()
{
  struct usb_bus *bus;
  struct usb_device *dev;
  
  usb_init ();
  usb_find_busses();
  usb_find_devices();
  for (bus = usb_get_busses (); bus; bus = bus->next)
    {
      for (dev = bus->devices; dev; dev = dev->next)
	{
	  int i;

	  for (i = 0; ids[i].vendor; i++)
	    if (dev->descriptor.idVendor == ids[i].vendor &&
		dev->descriptor.idProduct == ids[i].product)
	      {
		send_return (dev);
		break;
	      }
	  if (!ids[i].vendor)
	    printf ("Ignoring device 0x%04x:0x%04x\n", dev->descriptor.idVendor,
		    dev->descriptor.idProduct);
	}		       
    }		   

}
